--  file  : rfont.e
--  author: jiri babor
--  email : jbabor@paradise.net.nz
--  date  : 98-12-09

--------------------------------------------------------------------------
--  poor man's graphic fonts - similar to the popular putsxy: slightly  --
--  faster and a bit more flexible with transparent background option   --
--  it is self-initialized: it uses the built-in image of the ROM font  --
--------------------------------------------------------------------------

without type_check
include image.e
include machine.e

global constant
    CLEAR=-1,       -- transparent background
    HORIZONTAL=0,   -- line orientation
    VERTICAL=1
sequence char
integer
    bc,             -- text background color
    fc,             -- text foreground color
    orient,         -- text orientation
    x,              -- text pointer: left edge char space
    y               -- text pointer: top  edge char space

function rtrim(sequence s)
    -- trim (discard) trailing zeros of sequence s
    integer i
    i=length(s)
    while i do
        if s[i] then exit end if
        i=i-1
    end while
    return s[1..i]
end function

procedure load_rom_font()
    sequence b,cb,cbm,regs
    integer a
    regs = repeat(0,10)
    regs[REG_AX]=#1130
    regs[REG_BX]=#600
    regs=dos_interrupt(#10,regs)
    a=#10*regs[REG_ES]+regs[REG_BP]+16  -- bypass zero char
    char={}
    for i=1 to 255 do                   -- again bypass zero
        cbm={}
        cb=rtrim(peek({a,16}))          -- char bytes, right trimmed
        for j=1 to length(cb) do
            b=and_bits(cb[j],{128,64,32,16,8,4,2,1}) and 1
            cbm=append(cbm,rtrim(b))
        end for
        a=a+16
        char=append(char,cbm)
    end for
end procedure

global procedure setx(integer i)
    -- set horizontal position of text pointer
    x=i
end procedure

global procedure sety(integer i)
    -- set vertical position of text pointer
    y=i
end procedure

global function getx()
    -- return current horizontal position of text pointer
    return x
end function

global function gety()
    -- return current vertical position of text pointer
    return y
end function

global function background()
    -- return current background color (CLEAR = -1)
    return bc
end function

global function foreground()
    -- return current foreground color
    return fc
end function

global procedure set_background(integer c)
    -- set text background color
    bc=c
end procedure

global procedure set_foreground(integer c)
    -- set text foreground color
    fc=c
end procedure

global procedure set_orientation(integer o)
    -- set line orientation, horizontal or vertical
    orient=o
end procedure

global procedure write(sequence s)
    sequence c,cr,im
    integer l,n,u
    n=length(s)
    l=8*n
    if orient then      -- vertical print
        if bc=CLEAR then
            im=save_image({x,y-l+1},{x+15,y})
        else im=repeat(repeat(bc,16),l)
        end if
        u=l+1               -- local vertical text pointer
        for i=1 to n do
            c=char[s[i]]    -- trimmed char image
            for r=1 to length(c) do
                cr=c[r]     -- row of char image
                for j=1 to length(cr) do
                    if cr[j] then im[u-j][r]=fc end if
                end for
            end for
            u=u-8           -- index to next char
        end for
        display_image({x,y-l+1},im)
        y=y-l
    else                -- horizontal print
        if bc=CLEAR then
            im=save_image({x,y},{x+l-1,y+15})
        else im=repeat(repeat(bc,l),16)
        end if
        u=0                 -- local horizontal text pointer
        for i=1 to n do
            c=char[s[i]]    -- trimmed char image
            for r=1 to length(c) do
                cr=c[r]     -- row of char image
                for j=1 to length(cr) do
                    if cr[j] then im[r][u+j]=fc end if
                end for
            end for
            u=u+8           -- index to next char
        end for
        display_image({x,y},im)
        x=x+l
    end if
end procedure

-- initialize: defaults --------------------------------------------------------

load_rom_font()
x=0                 -- left edge of char space
y=0                 -- top edge of char space
bc=CLEAR            -- transparent background
fc=15               -- bright white foreground
orient=HORIZONTAL   -- default text direction
