-- file: stile.ex
-- author: Normand M. Blais
-- date:   Dec 26 1998

without warning
without trace
without type_check

include get.e  -- import: value()

include mcursor.e
include video.e
include rotseq.e
include keymap.e
include label.e

constant COLORS = {
GRAY,
BRIGHT_BLUE,
BRIGHT_BLUE,
BRIGHT_BLUE,
GRAY,
MAGENTA,
MAGENTA,
MAGENTA,
GRAY,
YELLOW,
YELLOW,
YELLOW,
GRAY,
BRIGHT_GREEN,
BRIGHT_GREEN,
BRIGHT_GREEN,
GRAY,
BRIGHT_RED,
BRIGHT_RED,
BRIGHT_RED,
GRAY
}

-- shapes
constant TA = {{50,0},{100,100},{0,100}}
constant TB = {{0,0},{100,0},{50,100}}
constant S1 = {{0,0},{100,0},{100,50},{0,50}}
constant S2 = {{50,0},{100,100},{50,125},{0,25}}
constant S3 = {{50,0},{100,25},{50,125},{0,100}}

constant WHEEL1 = {1,2,3,4,5,6,7,8,9,10,11,12},
         WHEEL2 = {13,14,15,16,17,18,19,20,21,10,11,12}

constant LEFT_WHEEL=1, RIGHT_WHEEL=2,
         LR_DIR=1, RL_DIR=2

sequence mid,left,right,topl,topr,botl,botr
sequence wheel1,wheel2
integer tot_moves

procedure init()
    -- create sections of puzzle
    mid  = {transXY(TA,270,107),transXY(S1,270,212),transXY(TB,270,267)}
    left = {transXY(TA,50,107),transXY(S1,50,212),transXY(TB,50,267)}
    right = {transXY(TA,490,107),transXY(S1,490,212),transXY(TB,490,267)}
    topl = {transXY(S2,105,80),transXY(TB,160,79),transXY(S3,215,80)}
    botl = {transXY(S3,105,270),transXY(TA,160,297),transXY(S2,215,270)}
    topr = {transXY(S2,325,80),transXY(TB,380,79),transXY(S3,435,80)}
    botr = {transXY(S3,325,270),transXY(TA,380,297),transXY(S2,435,270)}
    -- initialize the color arrays
    wheel1 = WHEEL1
    wheel2 = WHEEL2
    -- ????
    tot_moves = -1
    -- mode 18 640x480 16 colors
    video18()
    -- screen background
    polygon(WHITE,1,{{0,0},{639,0},{639,479},{0,479}})
    -- puzzle background
    polygon(BLACK,1,{{30,50},{612,50},{612,430},{30,430}})
    -- markers
    polygon(BRIGHT_RED,1,{{570,60},{600,60},{600,70},{570,70}})
    polygon(BRIGHT_GREEN,1,{{570,410},{600,410},{600,420},{570,420}})
    polygon(MAGENTA,1,{{40,60},{70,60},{70,70},{40,70}})
    polygon(BRIGHT_BLUE,1,{{40,410},{70,410},{70,420},{40,420}})
    -- left inner circles
    circle(210,237,100,GRAY,1)
    circle(210,237,98,BLACK,0)
    circle(210,237,54,BLACK,0)
    circle(210,237,34,BLACK,1)
    -- right inner circles
    circle(430,237,100,GRAY,1)
    circle(430,237,98,BLACK,0)
    circle(430,237,54,BLACK,0)
    circle(430,237,34,BLACK,1)
    -- labels:
        -- mouse directives
    label(70,450,"Turn (-)",MAGENTA)
    label(235,450,"Turn (+)",BRIGHT_BLUE)
    label(340,450,"Turn (-)",BRIGHT_RED)
    label(510,450,"Turn (+)",BRIGHT_GREEN)
        -- key commands
    label(380,5, "ESC    = Quit",BRIGHT_WHITE)
    label(380,25,"INSERT = New game",BRIGHT_WHITE)
    label(540,5, "F1 = Save",BRIGHT_WHITE)
    label(540,25,"F5 = Load",BRIGHT_WHITE)
        -- title
    label(30,15,"TURNSTILE PUZZLE",YELLOW)
    label(170,15,"by N.Blais",GREEN)
end procedure

procedure save()
    integer fn
    fn = open("stile.sav","w")
    for i=1 to length(wheel1) do
        printf(fn,"%d\n",wheel1[i])
    end for
    for i=1 to length(wheel2) do
        printf(fn,"%d\n",wheel2[i])
    end for
    printf(fn,"%d",tot_moves)
    close(fn)
end procedure

procedure load()
    object x
    sequence s
    integer fn
    fn = open("stile.sav","r")
    if fn = -1 then
        return 
    end if 
    s = {}
    while 1 do
        x = gets(fn)     
        if atom(x) then
            exit
        end if
        x = value(x)
        if sequence(x) then
            if x[1] = GET_SUCCESS then
                s = s & x[2]
            end if
        end if    
    end while
    for i=1 to length(wheel1) do
        wheel1[i] = s[i]
    end for
    for i=1 to length(wheel2) do
        wheel2[i] = s[i+length(wheel1)] 
    end for
    tot_moves = s[length(s)]
    close(fn)
end procedure

procedure show()       
    polygon(COLORS[wheel1[1]],1,botl[3]) 
    polygon(COLORS[wheel1[2]],1,botl[2]) 
    polygon(COLORS[wheel1[3]],1,botl[1]) 
    polygon(COLORS[wheel1[4]],1,left[3]) 
    polygon(COLORS[wheel1[5]],1,left[2]) 
    polygon(COLORS[wheel1[6]],1,left[1]) 
    polygon(COLORS[wheel1[7]],1,topl[1]) 
    polygon(COLORS[wheel1[8]],1,topl[2]) 
    polygon(COLORS[wheel1[9]],1,topl[3]) 
    polygon(COLORS[wheel1[10]],1,mid[1]) 
    polygon(COLORS[wheel1[11]],1,mid[2]) 
    polygon(COLORS[wheel1[12]],1,mid[3]) 
    polygon(COLORS[wheel2[1]],1,botr[1])     
    polygon(COLORS[wheel2[2]],1,botr[2]) 
    polygon(COLORS[wheel2[3]],1,botr[3]) 
    polygon(COLORS[wheel2[4]],1,right[3]) 
    polygon(COLORS[wheel2[5]],1,right[2]) 
    polygon(COLORS[wheel2[6]],1,right[1]) 
    polygon(COLORS[wheel2[7]],1,topr[3]) 
    polygon(COLORS[wheel2[8]],1,topr[2]) 
    polygon(COLORS[wheel2[9]],1,topr[1])
end procedure

procedure turn(integer wheel, integer direction)
    if wheel = LEFT_WHEEL then
        if direction = LR_DIR then
            wheel1 = rotate_rl(wheel1)
            wheel1 = rotate_rl(wheel1)
        elsif direction = RL_DIR then
            wheel1 = rotate_lr(wheel1)
            wheel1 = rotate_lr(wheel1)
        end if
        wheel2[10..12] = wheel1[10..12]
    elsif wheel = RIGHT_WHEEL then
        if direction = LR_DIR then
            wheel2 = rotate_lr(wheel2)
            wheel2 = rotate_lr(wheel2)
        elsif direction = RL_DIR then
            wheel2 = rotate_rl(wheel2)
            wheel2 = rotate_rl(wheel2)
        end if
        wheel1[10..12] = wheel2[10..12]
    end if
    if tot_moves != -1 then
        tot_moves = tot_moves + 1
    end if
end procedure                                   

procedure mix()
    integer w,d
    for i=1 to 2000 do
       w = remainder(rand(981),2) + 1
       d = remainder(rand(981),2) + 1
       if w = 1 then
           if d = 1 then
               turn(1,LR_DIR)
           elsif d = 2 then
               turn(1,RL_DIR)
           end if
       elsif w = 2 then
           if d = 1 then
               turn(2,LR_DIR)
           elsif d = 2 then
               turn(2,RL_DIR)
           end if
       end if
    end for
end procedure

function win()
    if tot_moves != -1 then
        if not compare(wheel1,WHEEL1) and not compare(wheel2,WHEEL2) then
            return 1
        else
            return 0    
        end if
    end if
    return 0
end function

procedure endit()
    end_video()
end procedure    

-----------------------------------------------------------------------------

constant NOTHING   =  -99,
         QUIT      = 1000,
         WHEEL1_LR = 1001,
         WHEEL1_RL = 1002,
         WHEEL2_LR = 1003,
         WHEEL2_RL = 1004,
         NEWGAME   = 1005

constant BOXES = {
{{30,430},{175,470}},  -- Turn wheel 1 R to L
{{175,430},{321,470}}, -- Turn wheel 1 L to R
{{321,430},{466,470}}, -- Turn wheel 2 R to L
{{466,430},{612,470}}  -- Turn wheel 2 L to L
}

function whereIsMouse(integer x, integer y)
    if x > BOXES[1][1][1] and x < BOXES[1][2][1] and
       y > BOXES[1][1][2] and y < BOXES[1][2][2] then
       return WHEEL1_RL
    elsif x > BOXES[2][1][1] and x < BOXES[2][2][1] and
          y > BOXES[2][1][2] and y < BOXES[2][2][2] then
       return WHEEL1_LR
    elsif x > BOXES[3][1][1] and x < BOXES[3][2][1] and
          y > BOXES[3][1][2] and y < BOXES[3][2][2] then
       return WHEEL2_RL
    elsif x > BOXES[4][1][1] and x < BOXES[4][2][1] and
          y > BOXES[4][1][2] and y < BOXES[4][2][2] then
       return WHEEL2_LR
    else
        return NOTHING
    end if
end function

procedure event_loop()
    object spare
    object me        
    integer k,what2do,gameover
    gameover = 0
    show()
    mouse_events(LEFT_UP+MOVE)
    Change_cursor(2)  -- 1=hand 2=arrow
    mouse_pointer(1)
    while 1 do
        spare = get_key()
        if spare != -1 then
            k = spare
            mouse_pointer(0)
            if k = ESC or k = 'q' then
                exit
            elsif k = LEFT_ARROW then
                turn(1,RL_DIR)
            elsif k = RIGHT_ARROW then
                turn(1,LR_DIR)
            elsif k = UP_ARROW then
                turn(2,LR_DIR)
            elsif k = DOWN_ARROW then
                turn(2,RL_DIR)
            elsif k = INSERT then
                tot_moves = 1
                mix()
                tot_moves = 0
            elsif k = 's' or k = F1 then
                save()
            elsif k = 'l' or k = F5 then
                load()
            end if
            show()
            mouse_pointer(1)
        end if
        me = get_mouse()
        if sequence(me) then  
            if (me[2] > BOXES[1][1][1] and me[2] < BOXES[4][2][1] and  
                me[3] > BOXES[1][1][2] and me[3] < BOXES[4][2][2] ) then
                Change_cursor(1)
            else
                Change_cursor(2)
            end if
            if and_bits(me[1], LEFT_UP) then
                what2do = whereIsMouse(me[2],me[3])
                if what2do = WHEEL1_RL and not gameover then
                    mouse_pointer(0)
                    turn(1,RL_DIR)
                    mouse_pointer(1)          
                elsif what2do = WHEEL1_LR and not gameover then
                    mouse_pointer(0)
                    turn(1,LR_DIR)
                    mouse_pointer(1)          
                elsif what2do = WHEEL2_RL  and not gameover then
                    mouse_pointer(0)
                    turn(2,RL_DIR)
                    mouse_pointer(1)          
                elsif what2do = WHEEL2_LR and not gameover then
                    mouse_pointer(0)
                    turn(2,LR_DIR)
                    mouse_pointer(1)          
--                elsif what2do = NEWGAME then
--                    mouse_pointer(0)
--                    tot_moves = 1
--                    mix()
--                    tot_moves = 0
--                    mouse_pointer(1)          
--                elsif what2do = QUIT then
--                    exit
                elsif what2do = NOTHING then
                    -- DoNothing() 
                end if
                if tot_moves > 0 then
                     gameover = win()
                     if gameover then
                        k = wait_key()
                        exit
                     end if
                end if
                show()
            end if
        end if
    end while
    mouse_pointer(0)
end procedure

procedure main()
    init()
    event_loop()
    endit() 
end procedure   

main()


